@@ -23,16 +23,17 @@ import java.io.InputStream;
 import java.io.OutputStream;
 import java.util.ArrayList;
 import java.util.List;
+import java.util.function.Function;
 import java.util.zip.GZIPOutputStream;
 
 import arq.cmdline.ModContext;
 import arq.cmdline.ModLangOutput;
 import arq.cmdline.ModLangParse;
 import arq.cmdline.ModTime;
-import org.apache.jena.Jena;
 import org.apache.jena.atlas.io.IO;
 import org.apache.jena.atlas.lib.IRILib;
 import org.apache.jena.atlas.lib.Pair;
+import org.apache.jena.atlas.logging.Log;
 import org.apache.jena.cmd.ArgDecl;
 import org.apache.jena.cmd.CmdException;
 import org.apache.jena.cmd.CmdGeneral;
@@ -44,14 +45,12 @@ import org.apache.jena.rdfs.SetupRDFS;
 import org.apache.jena.riot.*;
 import org.apache.jena.riot.lang.LabelToNode;
 import org.apache.jena.riot.lang.StreamRDFCounting;
-import org.apache.jena.riot.system.ErrorHandlerFactory;
-import org.apache.jena.riot.system.StreamRDF;
-import org.apache.jena.riot.system.StreamRDFLib;
-import org.apache.jena.riot.system.StreamRDFWriter;
+import org.apache.jena.riot.system.*;
 import org.apache.jena.riot.tokens.Tokenizer;
 import org.apache.jena.riot.tokens.TokenizerText;
 import org.apache.jena.sparql.core.DatasetGraph;
 import org.apache.jena.sparql.core.DatasetGraphFactory;
+import org.apache.jena.sparql.core.Quad;
 import org.apache.jena.sys.JenaSystem;
 
 /** Common framework for running RIOT parsers */
@@ -62,11 +61,12 @@ public abstract class CmdLangParse extends CmdGeneral {
     protected ModTime modTime = new ModTime();
     protected ModLangParse modLangParse = new ModLangParse();
     protected ModLangOutput modLangOutput = new ModLangOutput();
-    protected SetupRDFS setup = null;
+    protected SetupRDFS setupRDFS = null;
     protected ModContext modContext = new ModContext();
-    protected ArgDecl strictDecl = new ArgDecl(ArgDecl.NoValue, "strict");
+    protected ArgDecl argStrict = new ArgDecl(ArgDecl.NoValue, "strict");
 
     protected boolean cmdStrictMode = false;
+    // Merge quads to triples.
 
     protected CmdLangParse(String[] argv) {
         super(argv);
@@ -74,9 +74,7 @@ public abstract class CmdLangParse extends CmdGeneral {
         addModule(modTime);
         addModule(modLangOutput);
         addModule(modLangParse);
-
-        super.modVersion.addClass(Jena.class);
-        // super.modVersion.addClass(RIOT.class) ;
+        super.modVersion.addClass(RIOT.class);
     }
 
     @Override
@@ -85,24 +83,51 @@ public abstract class CmdLangParse extends CmdGeneral {
     }
 
     protected List<ParseRecord> outcomes = new ArrayList<>();
-
     protected OutputStream outputWrite = System.out;
-    protected StreamRDF outputStream = null;
+    protected StreamRDF parserOutputStream = null;
     protected String parserBaseIRI = null;
     protected boolean passRelativeURIs = false;
     protected String writerBaseIRI = null;
 
     @Override
     protected void processModulesAndArgs() {
-        cmdStrictMode = super.contains(strictDecl);
+        cmdStrictMode = super.contains(argStrict);
+
+        // checking.
+
+        // True if any input is quads
+        // False is unknown (stdin, no --syntax)
+        // Check will also be done during parsing.
+        boolean someQuadsInput = false;
+
+        if ( ! getPositional().isEmpty() || modLangParse.getLang() != null ) {
+            // otherwise the input is stdin and maybe triples or quads.
+            if ( modLangParse.getLang() != null ) {
+                someQuadsInput = RDFLanguages.isQuads(modLangParse.getLang());
+            } else {
+                for ( String fn :  super.getPositional() ) {
+                    Lang lang = RDFLanguages.filenameToLang(fn);
+                    if ( !RDFLanguages.isQuads(lang) ) {
+                        someQuadsInput = false;
+                        break;
+                    }
+                }
+            }
+        }
+
+        if ( someQuadsInput && ! isQuadsOutput() )
+            Log.warn(SysRIOT.getLogger(), "Quads syntax in the input files but triple output requested.");
     }
 
     protected interface PostParseHandler {
         void postParse();
     }
 
-    static class ParseRecord {
+    protected static class ParseRecord {
+        // Display name (filename as given on the command line)
         final String filename;
+        // Resolved filename as a URL string.
+        final String sourceURL;
         final boolean success;
         final long timeMillis;
         final long triples;
@@ -110,9 +135,10 @@ public abstract class CmdLangParse extends CmdGeneral {
         final long tuples = 0;
         final ErrorHandlerCLI errHandler;
 
-        public ParseRecord(String filename, boolean successful, long timeMillis, long countTriples, long countQuads,
+        public ParseRecord(String filename, String sourceURL, boolean successful, long timeMillis, long countTriples, long countQuads,
                            ErrorHandlerCLI errHandler) {
             this.filename = filename;
+            this.sourceURL = sourceURL;
             this.success = successful;
             this.timeMillis = timeMillis;
             this.triples = countTriples;
@@ -121,6 +147,19 @@ public abstract class CmdLangParse extends CmdGeneral {
         }
     }
 
+    /** Quads to triples. */
+    protected static class QuadsToTriples extends StreamRDFWrapper {
+
+        public QuadsToTriples(StreamRDF destination) {
+            super(destination);
+        }
+
+        @Override
+        public void quad(Quad quad) {
+            super.triple(quad.asTriple());
+        }
+    }
+
     @Override
     protected void exec() {
         boolean oldStrictValue = SysRIOT.isStrictMode();
@@ -142,7 +181,7 @@ public abstract class CmdLangParse extends CmdGeneral {
     protected void exec$() {
 
         if ( modLangParse.getRDFSVocab() != null )
-            setup = RDFSFactory.setupRDFS(modLangParse.getRDFSVocab().getGraph());
+            setupRDFS = RDFSFactory.setupRDFS(modLangParse.getRDFSVocab().getGraph());
 
         if ( modLangOutput.compressedOutput() ) {
             try {
@@ -152,16 +191,36 @@ public abstract class CmdLangParse extends CmdGeneral {
             }
         }
 
-        outputStream = null;
+        parserOutputStream = null;
         PostParseHandler postParse = null;
 
-        outputStream = createStreamSink();
-        if ( outputStream == null ) {
+        parserOutputStream = createStreamSink();
+        if ( parserOutputStream == null ) {
             Pair<StreamRDF, PostParseHandler> p = createAccumulateSink();
-            outputStream = p.getLeft();
+            parserOutputStream = p.getLeft();
             postParse = p.getRight();
         }
 
+        if ( ! modLangParse.mergeQuads() && ! isQuadsOutput() ) {
+            // Only pass through triples.
+            if ( isStreamingOutput() ) {
+                Function<Quad, StreamTriplesOnly.QuadPolicy> action = (quad) -> {
+                    // dest may be significantly buffered over the top of the output stream.
+                    // The log message does not necessarily come out in the right place - it may be early.
+                    // "Best effort" attempt to align log message to output.
+                    IO.flush(outputWrite);
+                    Log.warn(SysRIOT.getLogger(), "Quads in triples output - quads ignored.");
+                    return StreamTriplesOnly.QuadPolicy.IGNORE;
+                };
+                parserOutputStream = StreamTriplesOnly.setActionIfQuads(parserOutputStream, action);
+            }
+            // If not streaming, leave to the writer,
+        }
+
+        // If QuadsToTriples is added here, then counts will be "triples only"
+        if (false && modLangParse.mergeQuads() )
+            parserOutputStream = new QuadsToTriples(parserOutputStream);
+
         try {
             // The actual parsing ...
             if ( super.getPositional().isEmpty() ) {
@@ -256,7 +315,8 @@ public abstract class CmdLangParse extends CmdGeneral {
             builder.lang(lang);
         }
 
-        // Set the source.
+        // Set the display name and the source URL.
+        String sourceURL = filename;
         if ( filename.equals("-") ) {
             if ( baseParserIRI == null ) {
                 baseParserIRI = "http://base/";
@@ -271,17 +331,18 @@ public abstract class CmdLangParse extends CmdGeneral {
                 // File handling will reverse the transformation to open
                 // the file correctly but for base name generation we want the %20
                 // form.
-                filename = IRILib.filenameToIRI(filename);
-            builder.source(filename);
+                sourceURL = IRILib.filenameToIRI(filename);
+            builder.source(sourceURL);
         }
-        return parseRIOT(builder, filename);
+        return parseRIOT(builder, filename, sourceURL);
     }
 
     // Return the default (fall-back) language used if no other choice is made.
     // Contrast with --syntax=.. which forces the language.
     protected abstract Lang dftLang();
 
-    protected ParseRecord parseRIOT(RDFParserBuilder builder, String filename) {
+    /** Parse one source */
+    protected ParseRecord parseRIOT(RDFParserBuilder builder, String filename, String sourceURL) {
         boolean checking = true;
         if ( modLangParse.explicitChecking() )
             checking = true;
@@ -304,10 +365,6 @@ public abstract class CmdLangParse extends CmdGeneral {
         builder.errorHandler(errHandler);
 
         // Make into a cmd flag. (input and output subflags?)
-        // If input is "label, then output using NodeToLabel.createBNodeByLabelRaw()
-        // ;
-        // else use NodeToLabel.createBNodeByLabel() ;
-        // Also, as URI.
         final boolean labelsAsGiven = false;
 // NodeToLabel labels = SyntaxLabels.createNodeToLabel() ;
 // if ( labelsAsGiven )
@@ -315,10 +372,19 @@ public abstract class CmdLangParse extends CmdGeneral {
         if ( labelsAsGiven )
             builder.labelToNode(LabelToNode.createUseLabelAsGiven());
 
-        StreamRDF s = outputStream;
-        if ( setup != null )
-            s = RDFSFactory.streamRDFS(s, setup);
-        StreamRDFCounting sink = StreamRDFLib.count(s);
+        // Build parser output additions.
+        StreamRDF s = parserOutputStream;
+        if ( setupRDFS != null ) {
+            // Remove literals as subjects
+            s = RDFSFactory.removeGeneralizedRDF(s);
+            // Generate RDFS (this feeds into the stream created above).
+            s = RDFSFactory.streamRDFS(s, setupRDFS);
+            // Parser sends data to RDFS, which goes to the filter, then to parserOutputStream
+        }
+        // If added here, count is quads and triples seen in the input.
+        if ( modLangParse.mergeQuads() )
+            s = new QuadsToTriples(s);
+        StreamRDFCounting parserOut = StreamRDFLib.count(s);
         s = null;
 
         boolean successful = true;
@@ -326,8 +392,8 @@ public abstract class CmdLangParse extends CmdGeneral {
         modTime.startTimer();
         RDFParser parser = builder.build();
         try {
-            sink.start();
-            parser.parse(sink);
+            parserOut.start();
+            parser.parse(parserOut);
             successful = true;
         } catch (RiotNotFoundException ex) {
             errHandler.error(ex.getMessage(), -1, -1);
@@ -337,9 +403,9 @@ public abstract class CmdLangParse extends CmdGeneral {
         } catch (IRIException ex) {
             successful = false;
         }
-        sink.finish();
+        parserOut.finish();
         long x = modTime.endTimer();
-        ParseRecord outcome = new ParseRecord(filename, successful, x, sink.countTriples(), sink.countQuads(), errHandler);
+        ParseRecord outcome = new ParseRecord(filename, sourceURL, successful, x, parserOut.countTriples(), parserOut.countQuads(), errHandler);
         return outcome;
     }
 
@@ -351,7 +417,6 @@ public abstract class CmdLangParse extends CmdGeneral {
         RDFFormat fmt = modLangOutput.getOutputStreamFormat();
         if ( fmt == null )
             return null;
-        /** Create an accumulating output stream for later pretty printing */
         return StreamRDFWriter.getWriterStream(outputWrite, fmt, RIOT.getContext());
     }
 
@@ -365,8 +430,15 @@ public abstract class CmdLangParse extends CmdGeneral {
             builder.format(fmt);
             if ( RDFLanguages.isQuads(fmt.getLang()) )
                 builder.source(dsg);
-            else
+            else {
+                // Should only see triples - this is a consistency check.
+                if ( dsg.size() > 0 ) {
+                    Log.error(SysRIOT.getLogger(), "Quads seen in input but output is triples only.");
+                    throw new CmdException();
+                }
                 builder.source(dsg.getDefaultGraph());
+            }
+
             String baseURI = writerBaseIRI;
             if ( baseURI != null )
                 builder.base(baseURI);
@@ -375,6 +447,30 @@ public abstract class CmdLangParse extends CmdGeneral {
         return Pair.create(sink, handler);
     }
 
+    protected boolean isQuadsOutput() {
+        // Use stream in preference - CmdLangParse
+        RDFFormat fmt = outputFormat();
+        // RDFLanguages.isTriples means the language can be used in a triples context
+        // hence the test is "not quads".
+        if ( fmt != null && ! RDFLanguages.isQuads(fmt.getLang()) )
+            return false;
+        else
+            return true;
+    }
+
+    private RDFFormat outputFormat() {
+        // Use stream in preference - CmdLangParse
+        RDFFormat fmt = modLangOutput.getOutputStreamFormat();
+        if ( fmt == null)
+            fmt = modLangOutput.getOutputFormatted();
+        // RDFLanguages.isTriples means the language can be used in a triples context
+        // hence the test is "not quads".
+        return fmt;
+    }
+    protected boolean isStreamingOutput() {
+        return modLangOutput.getOutputStreamFormat() != null;
+    }
+
     protected Tokenizer makeTokenizer(InputStream in) {
         Tokenizer tokenizer = TokenizerText.create().source(in).build();
         return tokenizer;
